-- BeneCast by Wintrow

-- Props to Skurel's original BeneCast
-- This version is nearly completely rebuilt and rewritten though
-- Makes clickable buttons that casts beneficial spells on party members and the current target

-- This lua-file is both to initialize BeneCast with crucial variables as well as constants used throughout the code
-- It also helps to see the whole BeneCast variable in one place, so some members are defined as nil

BeneCast = LibStub('AceAddon-3.0'):NewAddon('BeneCast','AceConsole-3.0','AceEvent-3.0','AceTimer-3.0');
-- Holds the localisation-table, see Localisation.lua for more
BeneCast.Localisation = LibStub("AceLocale-3.0"):GetLocale('BeneCast');

-- Holds the helper functions
BeneCast.Helper = {};
LibStub("LibHandyHelper"):Embed(BeneCast.Helper);

-- Holds the MindReader-library
BeneCast.MindReader = {};
LibStub("LibMindReader"):Embed(BeneCast.MindReader);

local L = BeneCast.Localisation;

BeneCast.Constants = {
	-- This is used to reset the DB if severely outdated
	DBVersion = 10.1,
	
	-- This is the version of the addon, shown at the config screen
	Version = GetAddOnMetadata('BeneCast', 'Version'),
	
	Name = GetAddOnMetadata('BeneCast', 'Title'),
	
	-- Possible targets to attach a panel to
	Targets = {	'player', 'party1', 'party2', 'party3', 'party4', 'pet', 'partypet1', 'partypet2', 'partypet3', 'partypet4', 'target', 'targettarget', 'focus' },
	
	-- How spells should be sorted
	TypesSorted = { 'other', 'rez', 'brez', 'heal', 'hot', 'consume', 'buff', 'cure' },
	
	-- To easily find the pet of a certain unit
	PetsByOwner = {
		player = 'pet',		
	},
	
	-- To easily find the owner of a certain pet
	OwnersByPet = {
		pet = 'player',
	},
	
	-- To find the next orientation when doing "change orientation" on a panel
	Orientations = {
		RIGHT = {
			translation = L['Right'],
			next = 'DOWN'
		},
		DOWN = {
			translation = L['Down'],
			next = 'LEFT',
		},
		LEFT = {
			translation = L['Left'],
			next = 'UP',
		},
		UP = {
			translation = L['Up'],
			next = 'RIGHT',
		},
	},
	
	-- All classes, don't think I'm actually using this anywhere though...
	Classes = { 'DEATHKNIGHT','DRUID','HUNTER','MAGE','PALADIN','PRIEST','ROGUE','SHAMAN','WARLOCK','WARRIOR' },
	
	-- Lists the targettypes that a config can list
	TargetTypes = {
		all       = { player = true, pet = true, party = true, partypet = true, other = true, trans = 'All' },
		allonly   = { pet = true, party = true, partypet = true, other = true, trans = 'AllOnly' },
		self      = { player = true, trans = 'Self' },
		pet       = { pet = true, trans = 'Pet' },
		group     = { player = true, pet = true, party = true, partypet = true, trans = 'Group' },
		grouponly = { pet = true, party = true, partypet = true, trans = 'GroupOnly' },
	},
	
	-- All configs
	Configs = { 'player', 'pet', 'party', 'partypet', 'other' },
};
for i = 1, 4 do
	BeneCast.Constants.PetsByOwner['party'..i] = 'partypet'..i;
	BeneCast.Constants.OwnersByPet['partypet'..i] = 'party'..i;
end

-- Holds the database for Setup and Profiles
-- Has empty members so that wv.Setup and such can point to them
BeneCast.db = {
	Setup = nil,
	Profiles = nil,
	CustomButtons = nil,
	CustomEffects = nil,
};

-- The workvars contains all the tables and variables the code uses that would otherwise be globals "floating around"
-- These do not need to be saved across sessions as they are built up every time
-- Some of them are initialized here to a certain value though. After all, it's Init.lua :p
BeneCast.WorkVars = {
	PlayerClass = '',
	AttachedTargets = {}, -- Array to hold the list of targets we are currently attached to
	                      -- A fix by Shimavak on the 8.xx-version that I carried through to this one, prevents overupdating a panel
	ButtonsThatFlash = {}, -- Holds all buttons that are supposed to flash
	UnitsToUpdate = { -- It is preferred that the code schedules the update of a frame to avoid overupdating
		count = 0,
	},
	ConsiderInCombat = false, -- Right before combat an event is fired that allows the addon to change stuff before the lockdown is in effect
	                          -- This var indicates that this is the case
	ButtonInfo = {}, -- Holds the currently defined button-definitions
	EffectInfo = {}, -- Holds the currently defined effect-definitions
	DispelInfo = { -- Holds the buttonnames that can dispel a certain effect
		poison = {},
		magic = {},
		curse = {},
		disease = {},
	},
	SpellInfo = {}, -- Holds the translated version of the spells-table in SpellInfo.lua
	GroupInfo = {}, -- Holds the translated version of the groups-table in SpellInfo.lua
	TempEnchantsInfo = {}, -- Holds a smaller version of the Spells-table with only the known enchants, for fast access
	LoadCustomButtons = false, -- Marker to know when to load the saved-variables into ButtonInfo, enabled in OnInitialize()
	LoadKnownSpells = false, -- Marker to know when to load known spells, enabled in SPELLS_LOADED event
	LoadPreloadButtons = true, -- Marker to know when to load the preloaded buttons
	Panels = {}, -- Holds all the created panels as wv.Panels[profileid][unit] = panel
	OldButtons = {}, -- When buttons are no longer needed they are kept here for future recycling purposes
	MHWeaponEnchant = nil, -- Holds info on what (known) enchant is on the Mainhand
	OHWeaponEnchant = nil, -- Holds info on what (known) enchant is on the Offhand
	CurrentStance = 'none', -- Holds info on what stance the player is (this limits the available spells)
	DefaultConfig = nil, -- Filled up when addon is loaded, holds template to reset the db if needed
	SnapToFound = {}, -- Holds the addons that are loaded and exist in the snapto-table in SnapTo.lua
	AttachFrames = nil, -- Holds a variable to remember to re-attach all frames at the next convenience
	DisabledTimers = {}, -- Stores timers that were stopped when addon was disabled
	DefaultOptions = nil, -- Filled up when the addon is in it's first execution, holds template for when options need to be rebuilt
	Options = nil, -- Refilled on each refresh, holds actual optionstable for cmdline and GUI
	MinimapButton = nil, -- Holds the minimapbutton, see Minimap.lua for more
	Setup = BeneCast.db.Setup, -- Shortcut to the db
	Profiles = BeneCast.db.Profiles, -- Shortcut to the db
	CustomButtons = BeneCast.db.CustomButtons, -- Shortcut to the db
	CustomEffects = BeneCast.db.CustomEffects, -- Shortcut to the db
	GUIState = {  -- Holds the methods to be copied to below objects
		defaulticon = 'Interface\\Icons\\UI-Quickslot',
		previous = nil,
		previousicon = nil,
		current = nil,
		currenticon = nil,
		next = nil,
		nexticon = nil,
		InfoTable = nil,
		indextable = nil,
		name_in_edit = false,
		currentitem = {
			changed = nil,
			origin = nil,
			name = nil,
			attribs = nil,
			target = nil,
			icon = nil,
		},
		template = nil,
	},
	CustomButtonGUIState = {}, -- Holds the state of the CustomButton-GUI
	CustomEffectGUIState = {}, -- Holds the state of the CustomEffect-GUI
};
BeneCast.WorkVars.CustomButtonGUIState.InfoTable = BeneCast.WorkVars.ButtonInfo;
BeneCast.WorkVars.CustomEffectGUIState.InfoTable = BeneCast.WorkVars.EffectInfo;

BeneCast.PreloadButtons = {
	assist = {
		origin = 'preload',
		name = L['Assist'],
		group = L['Assist'],
		target = 'allonly',
		attribs = { type = 'assist' },
		sorting = 1,
		type = 'other',
		icon = 'Interface\\Icons\\Ability_Marksmanship',
		class = nil, -- Can contain a table of the classes that can preload this button
	},
	focus = {
		origin = 'preload',
		target = 'all',
		name = L['Focus'],
		group = L['Focus'],
		type = 'other',
		attribs = { type = 'focus' },
		icon = 'Interface\\Icons\\Ability_CheapShot',
		sorting = 1,
	},
};

BeneCast.PreloadEffects = {

};

BeneCast.ButtonTemplates = {
	spell = {
		origin = 'custom',
		target = 'all',
		name = 'Spell',
		group = 'Spell',
		type = 'buff',
		attribs = { type = 'spell', spell = 'spellname' },
		icon = 'Interface\\Icons\\INV_Misc_QuestionMark',
	},
};

-- Holds the template for a panel, see Panel.lua for more
BeneCast.Panel = {
	unit = nil,
	pf = nil,
	buttons = {},
	DropDown = {}, -- We can't set this to nil because BeneCastPanelDropDown refers to it
};

-- Holds the template for a button, see Button.lua for more
BeneCast.Button = {
	NormalTexture = nil,
	Cooldown = nil,
	Fade = nil,
	Range = nil,
	Duration = nil,
	unit = nil,
	pf = nil,
	seqno = nil,
};

-- Holds the timers, see Timer.lua for more
BeneCast.Timers = nil;

-- Holds the snapto-configs, SnapTo.lua for more
BeneCast.SnapTo = {};

-- Some handy shortcuts, to be copied in the header of every .lua
-- Makes the code shorter while keeping the main variable readable
--[[
local wv = BeneCast.WorkVars;
local BeneCastButton = BeneCast.Button;
local BeneCastPanel = BeneCast.Panel;
local BeneCastPanelDropDown = BeneCast.Panel.DropDown;
local hp = BeneCast.Helper;
local L = BeneCast.Localisation;
local const = BeneCast.Constants;
]]--